import { GuildMember, User } from "discord.js";

import { apiRequest } from "./execution";

export interface userQuotaData {
  /** How much quota does the user have */
  quota: number;
  /** How much used quota does the user have */
  used: number;
  /** Name of the quota unit */
  unitName?: string;
  /** String representation of quota */
  toString(): string;
}

export interface userQuotaRecoveryData extends userQuotaData {
  /** Timestamp until asked QuotaRecovery will get fully recovered, Infinity if never */
  recoveryTimestamp?: number;
}
/**
 * Represents an quota engine that instance could use
 */
export default interface IQuota {
  /**
   * Gets user remining quota (or lack of it)
   * @param request The request data
   */
  checkUser(
    user: User | GuildMember | string,
    request: apiRequest
  ): Promise<userQuotaData>;
  //checkUser(user: User | GuildMember | string, request: RequestMessage): userQuotaData;
  /** Name of the Quota engine */
  name?: string;
  /**
   * Checks recovery of quota units for a given user
   * @param user the user to check
   * @param request The request data
   * @param unitCount for how many units to check for recovery
   */
  getUserQuotaRecovery(
    user: User | GuildMember | string,
    request: apiRequest,
    unitCount?: number
  ): Promise<userQuotaRecoveryData>;
}
