import { ApplicationCommandType, ChatInputCommandInteraction, APIApplicationCommandOption, ApplicationCommandOptionType, APIEmbedField } from "discord.js";

import
  Command
,{ApplicationIntegrationType
, InteractionContextTypes
} from "../command";
import { config } from "../index";

export default class MyLimit extends Command implements Command {
  name = "check-quota";
  description = "Checks your quota and usage";
  type = ApplicationCommandType.ChatInput;
  options: APIApplicationCommandOption[] = [
    {
      name: "recovery-for",
      description: "Get the recovery time for given quota units count (default: until can use the bot or 1)",
      type: ApplicationCommandOptionType.Integer,
      required: false,
    },
    {
      name: "ephemeral",
      description: "if true, only you can see the response (default true)",
      type: ApplicationCommandOptionType.Boolean,
      required: false,
    }
  ];
  integration_types = [
    ApplicationIntegrationType.Guild_Install,
    ApplicationIntegrationType.User_Install
  ];
  contexts = [
    InteractionContextTypes.Guild,
    InteractionContextTypes.BotDM,
    InteractionContextTypes.PrivateChannel
  ];

  async execute(interaction: ChatInputCommandInteraction) {
    let recoveryFor = interaction.options.getInteger("recovery-for", false);
    const ephemeral = interaction.options.getBoolean("ephemeral", false) ?? true;

    if (recoveryFor === null) {
      const userQuota = await config.quota.checkUser(
        interaction.user,
        interaction,
      );
      const remaining = userQuota.quota - userQuota.used;
      // NOTE: In order for user to execute the bot again, the used must be lower than quota
      //       user can't execute if quota and used are equal
      //       thus why we add 1 here
      recoveryFor = remaining >= 0 ? 1 : 1 - remaining;
    }
    if (recoveryFor <= 0) recoveryFor = 1;

    const userQuotaRecovery = await config.quota.getUserQuotaRecovery(interaction.user, interaction, recoveryFor);

    const fields: APIEmbedField[] = [];

    fields.push({ name: "Quota", inline: true, value: `${userQuotaRecovery.quota} ${userQuotaRecovery.unitName}`.trim() });
    fields.push({ name: "Usage", inline: true, value: `${userQuotaRecovery.used} ${userQuotaRecovery.unitName}`.trim() });

    if (userQuotaRecovery.recoveryTimestamp !== undefined) fields.push({
      name: `Recovery for ${recoveryFor} ${userQuotaRecovery.unitName}`.trim(),
      value: userQuotaRecovery.recoveryTimestamp === Infinity ? "never" :
        `<t:${Math.ceil(userQuotaRecovery.recoveryTimestamp/1000)}:R>`
    });

    await interaction.reply({
      embeds: [{
        author: { name: interaction.user.username, icon_url: interaction.user.displayAvatarURL({ size: 128 }) },
        fields: fields,
      }],
      ephemeral: ephemeral,
    });
  }
}
